import {
  __spreadProps,
  __spreadValues,
  presets
} from "./chunk-IVRS5NIF.mjs";

// src/index.ts
import { promises as fs } from "fs";
import { createUnplugin } from "unplugin";

// src/core/options.ts
import { resolve } from "path";
import { toArray } from "@antfu/utils";
import { createFilter } from "@rollup/pluginutils";
function resolveOptions(options = {}) {
  const imports = flattenImportsMap(options.imports);
  const resolved = __spreadProps(__spreadValues({}, options), {
    dts: options.dts === false ? false : resolve(options.dts || "global-imports.d.ts"),
    imports,
    matchRE: new RegExp(`\\b(${Object.keys(imports).join("|")})\\b`, "g"),
    idFilter: createFilter(options.include || [/\.[jt]sx?$/, /\.vue\??/], options.exclude || [/node_modules/, /\.git/])
  });
  return resolved;
}
function flattenImportsMap(map) {
  const flat = {};
  toArray(map).forEach((definition) => {
    if (typeof definition === "string") {
      if (!presets[definition])
        throw new Error(`[global-import] preset ${definition} not found`);
      definition = presets[definition];
    }
    for (const mod of Object.keys(definition)) {
      for (const id of definition[mod]) {
        const meta = {
          module: mod
        };
        if (Array.isArray(id)) {
          meta.name = id[1];
          meta.from = id[0];
        } else {
          meta.name = id;
        }
        if (flat[meta.name])
          throw new Error(`[global-import] identifier ${meta.name} already defined with ${flat[meta.name].module}`);
        flat[meta.name] = meta;
      }
    }
  });
  return flat;
}

// src/core/transform.ts
var excludeRegex = [
  /\bimport\s*\{([\s\S]*?)\}\s*from\b/g,
  /\bfunction\s*([\s\S]+?)\s*\(/g,
  /\b(?:const|let|var)\s*([\w\d_$]+?)\b/g
];
function transform(code, id, { matchRE, imports }) {
  const matched = new Set(Array.from(code.matchAll(matchRE)).map((i) => i[1]));
  for (const regex of excludeRegex) {
    Array.from(code.matchAll(regex)).flatMap((i) => {
      var _a;
      return ((_a = i[1]) == null ? void 0 : _a.split(",")) || [];
    }).forEach((i) => matched.delete(i.trim()));
  }
  const modules = {};
  Array.from(matched).forEach((name) => {
    const moduleName = imports[name].module;
    if (!modules[moduleName])
      modules[moduleName] = [];
    modules[moduleName].push(imports[name]);
  });
  const importStatements = Object.entries(modules).map(([moduleName, names]) => {
    const imports2 = names.map(({ name, from }) => from ? `${from} as ${name}` : name).join(", ");
    return `import { ${imports2} } from '${moduleName}';`;
  }).join("");
  return importStatements + code;
}

// src/core/dts.ts
function generateDeclration(imports) {
  const body = Object.entries(imports).map(([name, info]) => `  const ${name}: typeof import('${info.module}')['${info.from || name}']`).join("\n");
  return `// Generated by 'unplugin-auto-import'
declare global {
${body}
}
export {}
`;
}

// src/index.ts
var src_default = createUnplugin((options) => {
  const resolved = resolveOptions(options);
  if (!Object.keys(resolved.imports).length)
    console.warn("[global-imports] plugin installed but no imports has defined, see https://github.com/antfu/unplugin-auto-import#configurations for configurations");
  if (resolved.dts)
    fs.writeFile(resolved.dts, generateDeclration(resolved.imports), "utf-8");
  return {
    name: "unplugin-auto-import",
    enforce: "post",
    transformInclude(id) {
      return resolved.idFilter(id);
    },
    transform(code, id) {
      return transform(code, id, resolved);
    }
  };
});

export {
  src_default
};
